@@ -3,6 +3,6 @@
 #ifndef BURN__DEBUG_H
 #define BURN__DEBUG_H
 
-void burn_print(int level, char *a, ...);
+void burn_print(int level, const char *a, ...);
 
 #endif /* BURN__DEBUG_H */
