@@ -1,11 +1,18 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
+/* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
+   Copyright (c) 2006 - 2013 Thomas Schmitt <scdbackup@gmx.net>
+   Provided under GPL version 2 or later.
+*/
+
 #ifndef BURN__INIT_H
 #define BURN__INIT_H
 
 extern int burn_running;
 
-/** Indicator for burn_drive_get_status() wether a signal hit parts of the
+extern double lib_start_time;
+
+/** Indicator for burn_drive_get_status() whether a signal hit parts of the
     thread team. 
     0= all works well ,
     1 to 5 = waiting for eventual signal on control thread
@@ -48,4 +55,9 @@ void *burn_alloc_mem(size_t size, size_t count, int flag);
 	}
 
 
+/* B20122 */
+int burn_grab_prepare_sig_action(int *signal_action_mem, int flag);
+int burn_grab_restore_sig_action(int signal_action_mem, int flag);
+
+
 #endif /* BURN__INIT_H */
