@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2014 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -22,6 +22,11 @@
 #include <fcntl.h>
 #include <errno.h>
 
+/* ts B41126 : O_BINARY is needed for Cygwin but undefined elsewhere */
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 #include "sector.h"
 #include "libburn.h"
 #include "drive.h"
@@ -281,7 +286,7 @@ void burn_packet_process(struct burn_drive *d, unsigned char *data,
 *//* write(o->datafd, data, 2352); */
 }
 
-/*  so yeah, when you uncomment these, make them write zeros insted of crap
+/*  so yeah, when you uncomment these, make them write zeros instead of crap
 static void write_empty_sector(int fd)
 {
 	static char sec[2352], initialized = 0;
@@ -309,8 +314,33 @@ static void flipq(unsigned char *sub)
 */
 
 
+/** @param flag bit1= be silent on failure
+                bit5= report failure with severity DEBUG
+*/
+static int burn_stdio_seek(int fd, off_t byte_address, struct burn_drive *d,
+                           int flag)
+{
+	char msg[80];
+
+	if (lseek(fd, byte_address, SEEK_SET) != -1)
+		return 1;
+	if (!(flag & 2)) {
+		sprintf(msg, "Cannot address start byte %.f",
+				(double) byte_address);
+				libdax_msgs_submit(libdax_messenger,
+			 	d->global_index, 0x00020147,
+				(flag & 32) ?
+				 LIBDAX_MSGS_SEV_DEBUG : LIBDAX_MSGS_SEV_SORRY,
+				LIBDAX_MSGS_PRIO_HIGH, msg, errno, 0);
+	}
+	return 0;
+}
+
+
 /* ts A70904 */
-/** @param flag bit0=be silent on data shortage */
+/** @param flag bit0= be silent on data shortage
+                bit5= report data shortage with severity DEBUG
+*/
 int burn_stdio_read(int fd, char *buf, int bufsize, struct burn_drive *d,
 			int flag)
 {
@@ -325,7 +355,9 @@ int burn_stdio_read(int fd, char *buf, int bufsize, struct burn_drive *d,
 	if(todo > 0 && !(flag & 1)) {
 		libdax_msgs_submit(libdax_messenger, d->global_index,
 			0x0002014a,
-			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
+			(flag & 32) ?
+			  LIBDAX_MSGS_SEV_DEBUG : LIBDAX_MSGS_SEV_SORRY,
+			 LIBDAX_MSGS_PRIO_HIGH,
 			"Cannot read desired amount of data", errno, 0);
 	}
 	if (count < 0)
@@ -334,11 +366,82 @@ int burn_stdio_read(int fd, char *buf, int bufsize, struct burn_drive *d,
 }
 
 
+/* With DVD and BD media, the minimum ECC entity is read instead of single
+   blocks.
+   @param flag see burn_read_data() in libburn.h
+*/
+static int retry_mmc_read(struct burn_drive *d, int chunksize, int sose_mem,
+                          int start, char **wpt, off_t *data_count,
+                          int flag)
+{
+	int i, err, todo;
+	int retry_at, retry_size;
+
+	retry_at = start;
+	retry_size = chunksize;
+	todo = chunksize;
+	retry_size = 16;                               /* DVD ECC block size */
+	if (d->current_is_cd_profile) {
+		retry_size = 1;                             /* CD block size */
+	} else if (d->current_profile >= 0x40 && d->current_profile <= 0x43) {
+		retry_size = 32;                          /* BD cluster size */
+	}
+	for (i = 0; todo > 0; i++) {
+		if (flag & 2)
+			d->silent_on_scsi_error = 1;
+		else if (flag & 32)
+			d->silent_on_scsi_error = 3;
+		retry_at = start + i * retry_size;
+		if (retry_size > todo)
+			retry_size = todo;
+		err = d->read_10(d, retry_at, retry_size, d->buffer);
+		if (flag & (2 | 32)) 
+			d->silent_on_scsi_error = sose_mem;
+		if (err == BE_CANCELLED)
+			return 0;
+		memcpy(*wpt, d->buffer->data, retry_size * 2048);
+		*wpt += retry_size * 2048;
+		*data_count += retry_size * 2048;
+		todo -= retry_size;
+	}
+	return 1;
+}
+
+
+/* @param flag see burn_read_data() in libburn.h
+*/
+static int retry_stdio_read(struct burn_drive *d, int fd, int chunksize,
+                            int start, char **wpt, off_t *data_count,
+                            int flag)
+{
+	int i, ret, to_read, todo;
+
+	ret = burn_stdio_seek(fd, ((off_t) start) * 2048, d, flag & 2);
+	if (ret <= 0)
+		return ret;
+	todo = chunksize * 2048;
+	for (i = 0; todo > 0; i += 2048) {
+		to_read = todo;
+		if (to_read > 2048)
+			to_read = 2048;
+		ret = burn_stdio_read(fd, (char *) d->buffer->data, to_read,
+					d, 1);
+		if (ret <= 0)
+			return 0;
+		memcpy(*wpt, d->buffer->data, to_read);
+		*wpt += to_read;
+		*data_count += to_read;
+		todo -= to_read;
+	}
+	return 1;
+}
+
+
 /* ts A70812 : API function */
 int burn_read_data(struct burn_drive *d, off_t byte_address,
                    char data[], off_t data_size, off_t *data_count, int flag)
 {
-	int alignment = 2048, start, upto, chunksize = 1, err, cpy_size, i;
+	int alignment = 2048, start, upto, chunksize = 1, err, cpy_size;
 	int sose_mem = 0, fd = -1, ret;
 	char msg[81], *wpt;
 	struct buffer *buf = NULL, *buffer_mem = d->buffer;
@@ -395,11 +498,11 @@ int burn_read_data(struct burn_drive *d, off_t byte_address,
 			sprintf(msg,
 			  "Read start address %ds larger than number of readable blocks %d",
 			  (int) (byte_address / 2048 + !!(byte_address % 2048)),
-			  d->media_read_capacity);
+			  d->media_read_capacity + 1);
 			libdax_msgs_submit(libdax_messenger, d->global_index,
-				0x00020172,
-				LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
-				msg, 0, 0);
+				0x00020172, (flag & 32) ?
+				LIBDAX_MSGS_SEV_DEBUG : LIBDAX_MSGS_SEV_SORRY,
+				LIBDAX_MSGS_PRIO_HIGH, msg, 0, 0);
 		}
 		{ret = 0; goto ex;}
 	}
@@ -423,7 +526,8 @@ int burn_read_data(struct burn_drive *d, off_t byte_address,
 		fd = d->stdio_fd;
 		if (fd < 0)
 			d->stdio_fd = fd =
-				open(d->devname, O_RDONLY | O_LARGEFILE);
+				open(d->devname,
+				     O_RDONLY | O_LARGEFILE | O_BINARY);
 		if (fd == -1) {
 			if (errno == EACCES && (flag & 2)) {
 				if (!(flag & 8))
@@ -433,29 +537,23 @@ int burn_read_data(struct burn_drive *d, off_t byte_address,
 					  LIBDAX_MSGS_PRIO_HIGH,
 			  "Failed to open device (a pseudo-drive) for reading",
 					  errno, 0);
-			} else if (errno!= ENOENT || !(flag & 2))
+			} else if (errno != ENOENT || !(flag & 2))
 				libdax_msgs_submit(libdax_messenger,
-				  d->global_index, 0x00020005,
-				  LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
+					d->global_index, 0x00020005,
+					(flag & 32) && errno == ENOENT ?
+				   		LIBDAX_MSGS_SEV_DEBUG :
+						LIBDAX_MSGS_SEV_SORRY,
+					LIBDAX_MSGS_PRIO_HIGH,
 			  "Failed to open device (a pseudo-drive) for reading",
-				   errno, 0);
+					errno, 0);
 			ret = 0;
 			if (errno == EACCES && (flag & 8))
 				ret= -2;
 			goto ex;
 		}
-		if (lseek(fd, byte_address, SEEK_SET) == -1) {
-			if (!(flag & 2)) {
-				sprintf(msg, "Cannot address start byte %.f",
-					(double) byte_address);
-				libdax_msgs_submit(libdax_messenger,
-				  d->global_index,
-				  0x00020147,
-				  LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
-				  msg, errno, 0);
-			}
-			ret = 0; goto ex;
-		}
+		ret = burn_stdio_seek(fd, byte_address, d, flag & (2 | 32));
+		if (ret <= 0)
+			goto ex;
 	}
 
 	d->busy = BURN_DRIVE_READING_SYNC;
@@ -475,6 +573,8 @@ int burn_read_data(struct burn_drive *d, off_t byte_address,
 			cpy_size = data_size - *data_count;
 		if (flag & 2)
 			d->silent_on_scsi_error = 1;
+		else if (flag & 32)
+			d->silent_on_scsi_error = 3;
 		if (flag & 16) {
 			d->had_particular_error &= ~1;
 			if (!d->silent_on_scsi_error)
@@ -484,51 +584,36 @@ int burn_read_data(struct burn_drive *d, off_t byte_address,
 			err = d->read_10(d, start, chunksize, d->buffer);
 		} else {
 			ret = burn_stdio_read(fd, (char *) d->buffer->data,
-						 cpy_size, d, !!(flag & 2));
+					      cpy_size, d,
+					      (flag & 32) | !!(flag & 2));
 			err = 0;
 			if (ret <= 0)
 				err = BE_CANCELLED;
 		}
-		if (flag & (2 | 16))
+		if (flag & (2 | 16 | 32))
 			d->silent_on_scsi_error = sose_mem;
 		if (err == BE_CANCELLED) {
 			if ((flag & 16) && (d->had_particular_error & 1))
 				{ret = -3; goto ex;}
-			/* Try to read a smaller part of the chunk */
-			if(!(flag & 4))
-			  for (i = 0; i < chunksize - 1; i++) {
-				if (flag & 2)
-					d->silent_on_scsi_error = 1;
-				if (d->drive_role == 1) {
-					err = d->read_10(d, start + i, 1,
-							 d->buffer);
-				} else {
-					ret = burn_stdio_read(fd,
-						(char *) d->buffer->data,
-						2048, d, 1);
-					if (ret <= 0)
-						err = BE_CANCELLED;
-				}
-				if (flag & 2) 
-					d->silent_on_scsi_error = sose_mem;
-				if (err == BE_CANCELLED)
-			break;
-				memcpy(wpt, d->buffer->data, 2048);
-				wpt += 2048;
-				*data_count += 2048;
+			/* Retry: with CD read by single blocks
+			          with other media: retry in full chunks
+			*/
+			if(flag & 4)
+				goto bad_read;
+			if (d->drive_role == 1) {
+				ret = retry_mmc_read(d, chunksize, sose_mem,
+						start, &wpt, data_count, flag);
+			} else {
+				ret = retry_stdio_read(d, fd, chunksize, 
+						start, &wpt, data_count, flag);
 			}
-			if (!(flag & 2))
-				libdax_msgs_submit(libdax_messenger,
-				  d->global_index,
-				  0x00020000,
-				  LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
-				  "burn_read_data() returns 0",
-				  0, 0);
-			ret = 0; goto ex;
+			if (ret <= 0)
+				goto bad_read;
+		} else {
+			memcpy(wpt, d->buffer->data, cpy_size);
+			wpt += cpy_size;
+			*data_count += cpy_size;
 		}
-		memcpy(wpt, d->buffer->data, cpy_size);
-		wpt += cpy_size;
-		*data_count += cpy_size;
 	}
 
 	ret = 1;
@@ -537,6 +622,14 @@ ex:;
 	d->buffer = buffer_mem;
 	d->busy = BURN_DRIVE_IDLE;
 	return ret;
+
+bad_read:;
+	if (!(flag & 2))
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+				0x00020000,
+				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
+				"burn_read_data() returns 0", 0, 0);
+	ret = 0; goto ex;
 }
 
 
@@ -599,6 +692,8 @@ int burn_read_audio(struct burn_drive *d, int sector_no,
 		cpy_size = chunksize * alignment;
 		if (flag & 2)
 			d->silent_on_scsi_error = 1;
+		else if (flag & 32)
+			d->silent_on_scsi_error = 3;
 		if (flag & 16) {
 			d->had_particular_error &= ~1;
 			if (!d->silent_on_scsi_error)
@@ -606,7 +701,7 @@ int burn_read_audio(struct burn_drive *d, int sector_no,
 		}
 		err = d->read_cd(d, start, chunksize, 1, 0x10, NULL, d->buffer,
 				 (flag & 8) >> 3);
-		if (flag & (2 | 16))
+		if (flag & (2 | 16 | 32))
 			d->silent_on_scsi_error = sose_mem;
 		if (err == BE_CANCELLED) {
 			if ((flag & 16) && (d->had_particular_error & 1))
@@ -615,9 +710,11 @@ int burn_read_audio(struct burn_drive *d, int sector_no,
 			  for (i = 0; i < chunksize - 1; i++) {
 				if (flag & 2)
 					d->silent_on_scsi_error = 1;
+				else if (flag & 32)
+					d->silent_on_scsi_error = 3;
 				err = d->read_cd(d, start + i, 1, 1, 0x10,
 				             NULL, d->buffer, (flag & 8) >> 3);
-				if (flag & 2)
+				if (flag & (2 | 32))
 					d->silent_on_scsi_error = sose_mem;
 				if (err == BE_CANCELLED)
 			  break;
