@@ -44,7 +44,7 @@ sg_initialize()         performs global initialization of the SCSI transport
                         facilities. Checks for compatibility of supporting
                         software components.
 
-sg_shutdown()           performs global finalizations and releases golbally
+sg_shutdown()           performs global finalizations and releases globally
                         acquired resources.
 
 sg_give_next_adr()      iterates over the set of potentially useful drive 
@@ -57,7 +57,7 @@ sg_dispose_drive()      finalizes adapter specifics of struct burn_drive
                         on destruction. Releases resources which were acquired
                         underneath scsi_enumerate_drives().
  
-sg_drive_is_open()      tells wether libburn has the given drive in use.
+sg_drive_is_open()      tells whether libburn has the given drive in use.
 
 sg_grab()               opens the drive for SCSI commands and ensures
                         undisturbed access.
@@ -65,7 +65,7 @@ sg_grab()               opens the drive for SCSI commands and ensures
 sg_release()            closes a drive opened by sg_grab()
 
 sg_issue_command()      sends a SCSI command to the drive, receives reply,
-                        and evaluates wether the command succeeded or shall
+                        and evaluates whether the command succeeded or shall
                         be retried or finally failed.
 
 sg_obtain_scsi_adr()    tries to obtain SCSI address parameters.
@@ -609,7 +609,7 @@ int sg_release(struct burn_drive *d)
 }
 
 
-/** Sends a SCSI command to the drive, receives reply and evaluates wether
+/** Sends a SCSI command to the drive, receives reply and evaluates whether
     the command succeeded or shall be retried or finally failed.
     Returned SCSI errors shall not lead to a return value indicating failure.
     The callers get notified by c->error. An SCSI failure which leads not to
@@ -768,7 +768,7 @@ int sg_obtain_scsi_adr(char *path, int *bus_no, int *host_no, int *channel_no,
 }
 
 
-/** Tells wether a text is a persistent address as listed by the enumeration
+/** Tells whether a text is a persistent address as listed by the enumeration
     functions.
 */
 int sg_is_enumerable_adr(char* adr)
