@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2016 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2020 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -277,7 +277,7 @@ struct burn_drive
 	               (which could need closing after write) */
 	int needs_close_session;
 	/* ts A71003 : whether a random write operation was done and no
-	               synchronize chache has happened yet */
+	               synchronize cache has happened yet */
 	int needs_sync_cache;
 
 	/* ts A80412 : whether to use WRITE12 with Streaming bit set
@@ -438,6 +438,17 @@ struct burn_drive
 	/* ts B90513 */
 	unsigned int write_retry_count;
 
+	/* ts C00806 */
+	/* 0=no change, 1=change, -1=already urged OS to revalidate medium */
+	int medium_state_changed;
+
+	/* ts C00822 */
+	/* If set, use Exact bit with SET STREAMING and use SET STREAMING
+	   even if the medium is a CD.
+	 */
+	int set_streaming_exact_bit;
+	int set_streaming_err;
+
 /* transport functions */
 	int (*grab) (struct burn_drive *);
 	int (*release) (struct burn_drive *);
