@@ -723,7 +723,7 @@ struct cue_sheet *burn_create_toc_entries(struct burn_write_opts *o,
 			runtime += 150;
 		} else if (pform != form) {
 
-		/* ts A70121 : This seems to be thw wrong test. Correct would
+		/* ts A70121 : This seems to be the wrong test. Correct would
 		   be to compare tar[]->mode or bit2 of ctladr.
 		*/ 
 
@@ -1904,6 +1904,9 @@ int burn_dvd_write_track(struct burn_write_opts *o,
 	/* ts A70213 : eventually expand size of track to max */
 	burn_track_apply_fillup(t, d->media_capacity_remaining, 0);
 
+	/* ts C00806 : Consider the track state changed by try to open */
+	d->medium_state_changed = 1;
+
 	if (d->current_profile == 0x11 || d->current_profile == 0x14 ||
 	    d->current_profile == 0x15) {
 		/* DVD-R, DVD-RW Sequential, DVD-R/DL Sequential */
@@ -2097,7 +2100,7 @@ int burn_dvd_write_session(struct burn_write_opts *o,
 		   is not readable.
 
 		   By default the open session gets closed here before the new
-		   session is written. E.g. after writing a small dummy seesion
+		   session is written. E.g. after writing a small dummy session
 		   number 2 one can read session 1 and write session 3 which
 		   points to data of session 1.
 
@@ -2762,30 +2765,57 @@ void burn_stdio_mmc_sync_cache(struct burn_drive *d)
 }
 
 
-/* ts A70912 */
-/* Enforces eventual nominal write speed.
-   @param flag bit0= initialize *prev_time */
-int burn_stdio_slowdown(struct burn_drive *d, struct timeval *prev_time,
-			 int amount, int flag)
+/* ts C00824 : API */
+/* Enforces nominal write speed */
+int burn_nominal_slowdown(int kb_per_second, int max_corr,
+			  struct timeval *prev_time,
+			  int *us_corr, off_t b_since_prev, int flag)
 {
 	struct timeval tnow;
-	double to_wait;
+	double to_wait, goal, corr;
+	int abs_max_corr;
 
 	if (flag & 1) {
 		gettimeofday(prev_time, NULL);
+		*us_corr = 0;
 		return 1;
 	}
-	if(d->nominal_write_speed <= 0)
+	if (kb_per_second <= 0)
 		return 2;
+
+	if (max_corr < -1.0e9 || max_corr > 1.0e9)
+		abs_max_corr = 1000000000;
+	else
+		abs_max_corr = abs(max_corr);
 	gettimeofday(&tnow, NULL);
-	to_wait = ( ((double) amount) / (double) d->nominal_write_speed ) - 
-		(double) ( tnow.tv_sec - prev_time->tv_sec ) -
-		(double) ( tnow.tv_usec - prev_time->tv_usec ) / 1.0e6
-		- 0.001; /* best would be 1 / kernel granularity HZ */
-	if (to_wait >= 0.0001) {
-		usleep((int) (to_wait * 1000000.0));
+	goal = ((double) b_since_prev) / 1000.0 / ((double) kb_per_second) +
+		((double) prev_time->tv_sec) +
+		((double) prev_time->tv_usec) / 1.0e6 +
+		((double) *us_corr) / 1.0e6 ;
+	to_wait = goal - ((double) tnow.tv_sec) -
+		  ((double) tnow.tv_usec) / 1.0e6;
+
+	/* usleep might be restricted to 999999 microseconds */
+	while (to_wait > 0.0) {
+		if (to_wait >= 0.5) {
+			usleep(500000);
+			to_wait -= 0.5;
+		} else if (to_wait >= 0.00001) {
+			usleep((int) (to_wait * 1000000.0));
+			to_wait = 0.0;
+		} else {
+			to_wait = 0.0;
+		}	
 	}
 	gettimeofday(prev_time, NULL);
+	corr = (goal - ((double) prev_time->tv_sec) -
+		((double) prev_time->tv_usec) / 1.0e6) * 1.0e6;
+	if (corr > abs_max_corr)
+		*us_corr = abs_max_corr;
+	else if (corr < -abs_max_corr)
+		*us_corr = -abs_max_corr;
+	else
+		*us_corr = corr;
 	return 1;
 }
 
@@ -2798,7 +2828,7 @@ int burn_stdio_write_track(struct burn_write_opts *o, struct burn_session *s,
 	struct burn_track *t = s->track[tnum];
 	struct burn_drive *d = o->drive;
 	char *buf = NULL;
-	int i, prev_sync_sector = 0;
+	int i, prev_sync_sector = 0, us_corr = 0, max_corr = 250000;
 	struct buffer *out = d->buffer;
 	struct timeval prev_time;
 
@@ -2818,7 +2848,10 @@ int burn_stdio_write_track(struct burn_write_opts *o, struct burn_session *s,
 	d->do_simulate = o->simulate;
 	d->sync_cache = burn_stdio_mmc_sync_cache;
 
-	burn_stdio_slowdown(d, &prev_time, 0, 1); /* initialize */
+	/* initialize */
+	burn_nominal_slowdown(d->nominal_write_speed, max_corr,
+				&prev_time, &us_corr, (off_t) 0, 1);
+
 	for (i = 0; open_ended || i < sectors; i++) {
 		/* transact a (CD sized) sector */
 		if (!sector_data(o, t, 0))
@@ -2831,14 +2864,25 @@ int burn_stdio_write_track(struct burn_write_opts *o, struct burn_session *s,
 		}
 		d->progress.sector++;
 		/* Flush to disk from time to time */
-		if (d->progress.sector - prev_sync_sector >=
-		    o->stdio_fsync_size && o->stdio_fsync_size > 0) {
-			prev_sync_sector = d->progress.sector;
-			if (!o->simulate)
-				burn_stdio_sync_cache(d->stdio_fd, d, 1);
+		if (o->stdio_fsync_size > 0) {
+			if (d->progress.sector - prev_sync_sector >=
+			    o->stdio_fsync_size) {
+				if (!o->simulate)
+					burn_stdio_sync_cache(d->stdio_fd, d,
+									 1);
+				burn_nominal_slowdown(
+					d->nominal_write_speed, max_corr,
+					&prev_time, &us_corr,
+					(off_t) (d->progress.sector -
+						 prev_sync_sector) *
+						 (off_t) 2048,
+					0);
+				prev_sync_sector = d->progress.sector;
+			}
+		} else if ((d->progress.sector % 512) == 0) {
+			burn_nominal_slowdown(d->nominal_write_speed, max_corr,
+				&prev_time, &us_corr, (off_t) (512 * 2048), 0);
 		}
-		if ((d->progress.sector % 512) == 0) 
-			burn_stdio_slowdown(d, &prev_time, 512 * 2, 0);
 	}
 
 	/* Pad up buffer to next full o->obs (usually 32 kB) */
@@ -3074,6 +3118,7 @@ return crap.  so we send the command, then ignore the result.
 		/* goto fail_wo_sync; */
 #endif /* Libburn_write_with_function_print_cuE */
 
+		d->medium_state_changed = 1;
 		ret = 1;
 		if (o->write_type == BURN_WRITE_SAO)
 			ret = d->send_cue_sheet(d, sheet);
